@@ -34,7 +34,7 @@ struct self_and_iter
 };
 
 signal_impl::signal_impl()
-: ref_count_(0), exec_count_(0), deferred_(0)
+: ref_count_(0), exec_count_(0), deferred_(false)
 {}
 
 // only MSVC needs this to guarantee that all new/delete are executed from the DLL module
@@ -67,12 +67,12 @@ void signal_impl::clear()
   slots_.clear();
 }
 
-signal_impl::size_type signal_impl::size() const
+signal_impl::size_type signal_impl::size() const noexcept
 {
   return slots_.size();
 }
 
-bool signal_impl::blocked() const
+bool signal_impl::blocked() const noexcept
 {
   for (const auto& slot : const_cast<const std::list<slot_base>&>(slots_))
   {
@@ -82,7 +82,7 @@ bool signal_impl::blocked() const
   return true;
 }
 
-void signal_impl::block(bool should_block)
+void signal_impl::block(bool should_block) noexcept
 {
   for (auto& slot : slots_)
   {
@@ -95,6 +95,11 @@ signal_impl::iterator_type signal_impl::connect(const slot_base& slot_)
   return insert(slots_.end(), slot_);
 }
 
+signal_impl::iterator_type signal_impl::connect(slot_base&& slot_)
+{
+  return insert(slots_.end(), std::move(slot_));
+}
+
 signal_impl::iterator_type signal_impl::erase(iterator_type i)
 {
   // Don't let signal_impl::notify() erase the slot. It would be more
@@ -119,6 +124,14 @@ signal_impl::iterator_type signal_impl::insert(signal_impl::iterator_type i, con
   return temp;
 }
 
+signal_impl::iterator_type signal_impl::insert(signal_impl::iterator_type i, slot_base&& slot_)
+{
+  auto temp = slots_.insert(i, std::move(slot_));
+  auto si = new self_and_iter(this, temp);
+  temp->set_parent(si, &notify);
+  return temp;
+}
+
 void signal_impl::sweep()
 {
   // The deletion of a slot may cause the deletion of a signal_base,
@@ -150,24 +163,24 @@ void* signal_impl::notify(void* d)
   }
   else                           // This is occuring during signal emission or slot erasure.
     si->self_->deferred_ = true; // => sweep() will be called from ~signal_exec() after signal emission.
-  return 0;                      // This is safer because we don't have to care about our
+  return nullptr;                // This is safer because we don't have to care about our
                                  // iterators in emit(), clear(), and erase().
 }
 
 } /* namespace internal */
 
-signal_base::signal_base()
+signal_base::signal_base() noexcept
 : impl_(nullptr)
 {}
 
-signal_base::signal_base(const signal_base& src)
+signal_base::signal_base(const signal_base& src) noexcept
 : trackable(),
   impl_(src.impl())
 {
   impl_->reference();
 }
 
-signal_base::signal_base(signal_base&& src) noexcept
+signal_base::signal_base(signal_base&& src)
 : trackable(std::move(src)),
   impl_(std::move(src.impl_))
 {
@@ -193,23 +206,23 @@ void signal_base::clear()
     impl_->clear();
 }
 
-signal_base::size_type signal_base::size() const
+signal_base::size_type signal_base::size() const noexcept
 {
   return (impl_ ? impl_->size() : 0);
 }
 
-bool signal_base::blocked() const
+bool signal_base::blocked() const noexcept
 {
   return (impl_ ? impl_->blocked() : true);
 }
 
-void signal_base::block(bool should_block)
+void signal_base::block(bool should_block) noexcept
 {
   if (impl_)
     impl_->block(should_block);
 }
 
-void signal_base::unblock()
+void signal_base::unblock() noexcept
 {
   if (impl_)
     impl_->block(false);
@@ -220,11 +233,21 @@ signal_base::iterator_type signal_base::connect(const slot_base& slot_)
   return impl()->connect(slot_);
 }
 
+signal_base::iterator_type signal_base::connect(slot_base&& slot_)
+{
+  return impl()->connect(std::move(slot_));
+}
+
 signal_base::iterator_type signal_base::insert(iterator_type i, const slot_base& slot_)
 {
   return impl()->insert(i, slot_);
 }
 
+signal_base::iterator_type signal_base::insert(iterator_type i, slot_base&& slot_)
+{
+  return impl()->insert(i, std::move(slot_));
+}
+
 signal_base::iterator_type signal_base::erase(iterator_type i)
 {
   return impl()->erase(i);
@@ -248,11 +271,22 @@ signal_base& signal_base::operator=(const signal_base& src)
   return *this;
 }
 
-signal_base& signal_base::operator=(signal_base&& src) noexcept
+signal_base& signal_base::operator=(signal_base&& src)
 {
-  trackable::operator=(std::move(src));
-  impl_ = std::move(src.impl_);
+  if (src.impl_ == impl_) return *this;
 
+  if (impl_)
+  {
+    // Disconnect all slots before impl_ is deleted.
+    // TODO: Move the signal_impl::clear() call to ~signal_impl() when ABI can be broken.
+    if (impl_->ref_count_ == 1)
+      impl_->clear();
+
+    impl_->unreference();
+  }
+
+  src.notify_callbacks();
+  impl_ = src.impl_;
   src.impl_ = nullptr;
 
   return *this;
