@@ -37,7 +37,7 @@ struct SIGC_API trackable_callback
 {
   void* data_;
   func_destroy_notify func_;
-  trackable_callback(void* data, func_destroy_notify func)
+  trackable_callback(void* data, func_destroy_notify func) noexcept
     : data_(data), func_(func) {}
 };
 
@@ -108,15 +108,20 @@ private:
  */
 struct SIGC_API trackable
 {
-  trackable();
+  // Concerning noexcept specifications:
+  // libsigc++ does not have complete control of what happens when notify_callbacks()
+  // is called. It may throw an exception. A method that calls notify_callbacks()
+  // shall not be declared noexcept.
 
-  trackable(const trackable& src);
+  trackable() noexcept;
 
-  trackable(trackable&& src) noexcept;
+  trackable(const trackable& src) noexcept;
+
+  trackable(trackable&& src);
 
   trackable& operator=(const trackable& src);
 
-  trackable& operator=(trackable&& src) noexcept;
+  trackable& operator=(trackable&& src);
 
   ~trackable();
 
